@@ -67,6 +67,8 @@ class FedMsgEncoder(json.encoder.JSONEncoder):
             return obj.__json__()
         if isinstance(obj, datetime.datetime):
             return time.mktime(obj.timetuple())
+        if isinstance(obj, time.struct_time):
+            return time.mktime(obj)
         if sqlalchemy:
             # As a last ditch, try using our sqlalchemy json encoder.
             sqla_type = sqlalchemy.ext.declarative.DeclarativeMeta
